@@ -30,6 +30,7 @@
 #define DEFAULT_REORDER_ITEMS                           TRUE
 #define DEFAULT_SKIP_ACTION_ON_KEY_DOWN                 FALSE
 #define DEFAULT_ADD_PRIMARY_CLIPBOARD                   FALSE
+#define DEFAULT_SHOW_QR_CODE                            FALSE
 #define DEFAULT_HISTORY_IGNORE_PRIMARY_CLIPBOARD        TRUE
 #define DEFAULT_ENABLE_ACTIONS                          FALSE
 
@@ -44,8 +45,9 @@
  * Selection for the popup command
  */
 
-#define XFCE_CLIPMAN_SELECTION    "XFCE_CLIPMAN_SELECTION"
-#define XFCE_CLIPMAN_MESSAGE      "MENU"
+#define XFCE_CLIPMAN_SELECTION        "XFCE_CLIPMAN_SELECTION"
+#define XFCE_CLIPMAN_MESSAGE          "MENU"
+#define XFCE_CLIPMAN_ACTION_MESSAGE   "ACTIONS"
 
 /*
  * Action Groups
