@@ -23,7 +23,6 @@
 #include <glib.h>
 #include <glib-object.h>
 #include <gtk/gtk.h>
-#include <exo/exo.h>
 #include <libxfce4util/libxfce4util.h>
 
 #include "common.h"
@@ -42,6 +41,7 @@ struct _ClipmanHistoryPrivate
 {
   GSList                       *items;
   const ClipmanHistoryItem     *item_to_restore;
+  const ClipmanHistoryItem     *primary_item_to_restore;
   guint                         max_texts_in_history;
   guint                         max_images_in_history;
   gboolean                      save_on_quit;
@@ -274,12 +274,13 @@ clipman_history_add_text (ClipmanHistory *history,
   item->content.text = g_strdup (text);
 
   /* Strip white spaces for preview */
-  tmp1 = g_strstrip (g_strdup (text));
-  while (g_strstr_len (tmp1, preview_length, "  "))
+  tmp1 = g_strchomp (g_strdup (text));
+
+  tmp2 = tmp1;
+  while (tmp2)
     {
-      tmp2 = exo_str_replace (tmp1, "  ", " ");
-      g_free (tmp1);
-      tmp1 = tmp2;
+      tmp2 = g_strchug(++tmp2);
+      tmp2 = g_strstr_len (tmp1, preview_length, "  ");
     }
 
   /* Shorten preview */
@@ -323,7 +324,7 @@ clipman_history_add_image (ClipmanHistory *history,
   item = g_slice_new0 (ClipmanHistoryItem);
   item->type = CLIPMAN_HISTORY_TYPE_IMAGE;
   item->content.image = gdk_pixbuf_copy (image);
-  item->preview.image = exo_gdk_pixbuf_scale_ratio (GDK_PIXBUF (image), 128);
+  item->preview.image = gdk_pixbuf_scale_simple (GDK_PIXBUF (image), 128, 128, GDK_INTERP_BILINEAR);
 
   DBG ("Copy of image (%p) is (%p)", image, item->content.image);
 
@@ -557,4 +558,3 @@ clipman_history_get_property (GObject *object,
       break;
     }
 }
-
