@@ -110,6 +110,7 @@ panel_plugin_register (XfcePanelPlugin *panel_plugin)
   xfce_panel_plugin_menu_show_about (panel_plugin);
   xfce_panel_plugin_menu_show_configure (panel_plugin);
   mi = gtk_check_menu_item_new_with_mnemonic (_("_Disable"));
+  gtk_widget_show (mi);
   xfce_panel_plugin_menu_insert_item (panel_plugin, GTK_MENU_ITEM (mi));
   xfconf_g_property_bind (plugin->channel, "/tweaks/inhibit",
                           G_TYPE_BOOLEAN, mi, "active");
@@ -135,16 +136,20 @@ static gboolean
 plugin_set_size (MyPlugin *plugin,
                  gint size)
 {
+#if !LIBXFCE4PANEL_CHECK_VERSION (4,13,0)
   GtkStyleContext *context;
   GtkBorder padding, border;
-  gint width, icon_width;
+  gint width;
   gint xthickness;
   gint ythickness;
+#endif
+  gint icon_size;
 
   size /= xfce_panel_plugin_get_nrows (plugin->panel_plugin);
-  gtk_widget_set_size_request(GTK_WIDGET(plugin->button), size, size);
-
-
+  gtk_widget_set_size_request (GTK_WIDGET (plugin->button), size, size);
+#if LIBXFCE4PANEL_CHECK_VERSION (4,13,0)
+  icon_size = xfce_panel_plugin_get_icon_size (XFCE_PANEL_PLUGIN (plugin->panel_plugin));
+#else
   /* Calculate the size of the widget because the theme can override it */
   context = gtk_widget_get_style_context (GTK_WIDGET (plugin->button));
   gtk_style_context_get_padding (context, gtk_widget_get_state_flags (GTK_WIDGET (plugin->button)), &padding);
@@ -156,17 +161,18 @@ plugin_set_size (MyPlugin *plugin,
   width = size - 2 * MAX (xthickness, ythickness);
 
   /* Since symbolic icons are usually only provided in 16px we
-  * try to be clever and use size steps */
+   * try to be clever and use size steps */
   if (width <= 21)
-    icon_width = 16;
+    icon_size = 16;
   else if (width >=22 && width <= 29)
-    icon_width = 24;
+    icon_size = 24;
   else if (width >= 30 && width <= 40)
-    icon_width = 32;
+    icon_size = 32;
   else
-    icon_width = width;
-  /* Adjust icon-size to panel size */
-  gtk_image_set_pixel_size (GTK_IMAGE (plugin->image), icon_width);
+    icon_size = width;
+#endif
+
+  gtk_image_set_pixel_size (GTK_IMAGE (plugin->image), icon_size);
 
   return TRUE;
 }
