@@ -38,19 +38,12 @@
 #include "history.h"
 #include "menu.h"
 
+
+
 /*
- * Popup command
+ * Private functions
  */
 
-static gboolean         my_plugin_set_popup_selection   (MyPlugin *plugin);
-static GdkFilterReturn  event_filter_popup_menu         (GdkXEvent *xevent,
-                                                         GdkEvent *event,
-                                                         MyPlugin *plugin);
-static gboolean         xfce_popup_grab_available       (GdkWindow *win,
-                                                         guint32 timestamp);
-
-
-
 static gboolean
 clipboard_manager_ownership_exists (void)
 {
@@ -70,6 +63,7 @@ MyPlugin *
 plugin_register (void)
 {
   MyPlugin *plugin = g_slice_new0 (MyPlugin);
+  GError *error = NULL;
 
   /* Locale */
   xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, NULL);
@@ -81,6 +75,25 @@ plugin_register (void)
       gsd_clipboard_manager_start (plugin->daemon, NULL);
     }
 
+  plugin->app = gtk_application_new ("org.xfce.clipman", 0);
+  g_application_register (G_APPLICATION (plugin->app), NULL, &error);
+  if (error != NULL)
+    {
+      g_warning ("Unable to register GApplication: %s", error->message);
+      g_error_free (error);
+      error = NULL;
+    }
+
+  if (g_application_get_is_remote (G_APPLICATION (plugin->app)))
+    {
+      g_message ("Primary instance org.xfce.clipman already running");
+      g_object_unref (plugin->app);
+      return FALSE;
+    }
+
+  g_set_application_name (_("Clipman"));
+  g_signal_connect_swapped (plugin->app, "activate", G_CALLBACK (plugin_popup_menu), plugin);
+
   /* Xfconf */
   xfconf_init (NULL);
   plugin->channel = xfconf_channel_new_with_property_base ("xfce4-panel", "/plugins/clipman");
@@ -133,8 +146,6 @@ plugin_register (void)
                             G_CALLBACK (plugin_save), plugin);
   g_signal_connect_swapped (plugin->history, "clear",
                             G_CALLBACK (plugin_save), plugin);
-  /* Set the selection for the popup command */
-  my_plugin_set_popup_selection (plugin);
 
   return plugin;
 }
@@ -280,7 +291,6 @@ plugin_free (MyPlugin *plugin)
       gsd_clipboard_manager_stop (plugin->daemon);
       g_object_unref (plugin->daemon);
     }
-  gdk_window_remove_filter (gtk_widget_get_window(plugin->menu), (GdkFilterFunc) event_filter_popup_menu, plugin);
   gtk_widget_destroy (plugin->menu);
   g_object_unref (plugin->channel);
   g_object_unref (plugin->actions);
@@ -293,6 +303,7 @@ plugin_free (MyPlugin *plugin)
   gtk_widget_destroy (plugin->popup_menu);
 #endif
 
+  g_object_unref (plugin->app);
   g_slice_free (MyPlugin, plugin);
   xfconf_shutdown ();
 }
@@ -352,187 +363,33 @@ plugin_configure (MyPlugin *plugin)
 void
 plugin_popup_menu (MyPlugin *plugin)
 {
+  if (xfconf_channel_get_bool (plugin->channel, "/tweaks/popup-at-pointer", FALSE))
+    {
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+      gtk_menu_popup (GTK_MENU (plugin->menu), NULL, NULL,
+                      NULL, NULL,
+                      0, gtk_get_current_event_time ());
+G_GNUC_END_IGNORE_DEPRECATIONS
+    }
+  else
+    {
 #ifdef PANEL_PLUGIN
-  gtk_menu_set_screen (GTK_MENU (plugin->menu), gtk_widget_get_screen (plugin->button));
-
-  if(!gtk_widget_has_grab(plugin->menu))
-  {
-    gtk_grab_add(plugin->menu);
-  }
-
-  gtk_menu_popup (GTK_MENU (plugin->menu), NULL, NULL,
-                  plugin->menu_position_func, plugin,
-                  0, gtk_get_current_event_time ());
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (plugin->button), TRUE);
-  xfce_panel_plugin_register_menu (plugin->panel_plugin, GTK_MENU (plugin->menu));
+      gtk_menu_set_screen (GTK_MENU (plugin->menu), gtk_widget_get_screen (plugin->button));
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+      gtk_menu_popup (GTK_MENU (plugin->menu), NULL, NULL,
+                      plugin->menu_position_func, plugin,
+                      0, gtk_get_current_event_time ());
+G_GNUC_END_IGNORE_DEPRECATIONS
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (plugin->button), TRUE);
+      xfce_panel_plugin_register_menu (plugin->panel_plugin, GTK_MENU (plugin->menu));
 #elif defined (STATUS_ICON)
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-  gtk_menu_set_screen (GTK_MENU (plugin->menu), gtk_status_icon_get_screen (plugin->status_icon));
+      gtk_menu_set_screen (GTK_MENU (plugin->menu),
+                           gtk_status_icon_get_screen (plugin->status_icon));
+      gtk_menu_popup (GTK_MENU (plugin->menu), NULL, NULL,
+                      plugin->menu_position_func, plugin->status_icon,
+                      0, gtk_get_current_event_time ());
 G_GNUC_END_IGNORE_DEPRECATIONS
-
-  if(!gtk_widget_has_grab(plugin->menu))
-  {
-    gtk_grab_add(plugin->menu);
+#endif
   }
-
-  gtk_menu_popup (GTK_MENU (plugin->menu), NULL, NULL,
-                  plugin->menu_position_func, plugin->status_icon,
-                  0, gtk_get_current_event_time ());
-#endif
-}
-
-/*
- * X11 Selection for the popup command
- */
-
-static gboolean
-my_plugin_set_popup_selection (MyPlugin *plugin)
-{
-  GdkScreen          *gscreen;
-  gchar              *selection_name;
-  Atom                selection_atom;
-  GtkWidget          *win;
-  Window              id;
-  Display            *display;
-  GdkWindow          *window;
-
-  win = gtk_invisible_new ();
-  gtk_widget_realize (win);
-  id = GDK_WINDOW_XID (gtk_widget_get_window (win));
-  display = gdk_x11_get_default_xdisplay ();
-
-  gscreen = gtk_widget_get_screen (win);
-  selection_name = g_strdup_printf (XFCE_CLIPMAN_SELECTION"%d",
-                                    gdk_screen_get_number (gscreen));
-  selection_atom = XInternAtom (display, selection_name, FALSE);
-  g_free(selection_name);
-
-  if (XGetSelectionOwner (display, selection_atom))
-    {
-      gtk_widget_destroy (win);
-      return FALSE;
-    }
-
-  XSelectInput (display, id, PropertyChangeMask);
-  XSetSelectionOwner (display, selection_atom, id, GDK_CURRENT_TIME);
-
-  window = gtk_widget_get_window (win);
-  gdk_window_add_filter (window, (GdkFilterFunc) event_filter_popup_menu, plugin);
-
-  return TRUE;
-}
-
-static GdkFilterReturn
-event_filter_popup_menu (GdkXEvent *xevent, GdkEvent *event, MyPlugin *plugin)
-{
-    XClientMessageEvent *evt;
-    GdkScreen *screen;
-    GdkWindow *root;
-    Atom message_type;
-    evt = (XClientMessageEvent *)xevent;
-
-    if (((XEvent *)xevent)->type != ClientMessage)
-      return GDK_FILTER_CONTINUE;
-
-    message_type = XInternAtom (gdk_x11_get_default_xdisplay (), "STRING", FALSE);
-    if (evt->message_type != message_type)
-      return GDK_FILTER_CONTINUE;
-
-    /* Copy workaround from xfdesktop to handle the awkward case where binding
-     * a keyboard shortcut to the popup command doesn't always work out... */
-#ifdef PANEL_PLUGIN
-    screen = gtk_widget_get_screen (GTK_WIDGET (plugin->button));
-#elif defined (STATUS_ICON)
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-    screen = gtk_status_icon_get_screen (plugin->status_icon);
-G_GNUC_END_IGNORE_DEPRECATIONS
-#endif
-    root = gdk_screen_get_root_window (screen);
-
-    if (!xfce_popup_grab_available (root, GDK_CURRENT_TIME))
-      {
-        g_critical ("Unable to get keyboard/mouse grab.");
-        return FALSE;
-      }
-
-  if (G_LIKELY (evt->format == 8) && (*(evt->data.b) != '\0'))
-    {
-
-      if (!g_ascii_strcasecmp (XFCE_CLIPMAN_MESSAGE, evt->data.b))
-        {
-          DBG ("Message received: %s", evt->data.b);
-
-          if (xfconf_channel_get_bool (plugin->channel, "/tweaks/popup-at-pointer", FALSE))
-            {
-              gtk_menu_popup (GTK_MENU (plugin->menu), NULL, NULL, NULL, NULL,
-                              0, gtk_get_current_event_time ());
-            }
-          else
-            {
-              plugin_popup_menu (plugin);
-            }
-
-          return TRUE;
-        } else if (!g_ascii_strcasecmp (XFCE_CLIPMAN_ACTION_MESSAGE, evt->data.b))
-        {
-             clipman_collector_show_actions();
-        }
-    }
-
-  return FALSE;
-}
-
-#if GTK_CHECK_VERSION (3, 20, 0)
-static void
-make_window_visible (GdkSeat *seat,
-                     GdkWindow *window,
-                     gpointer user_data)
-{
-  gdk_window_show (window);
-}
-#endif
-
-/* Code taken from xfwm4/src/menu.c:grab_available().  This should fix the case
- * where binding 'xfdesktop -menu' to a keyboard shortcut sometimes works and
- * sometimes doesn't.  Credit for this one goes to Olivier.
- */
-static gboolean
-xfce_popup_grab_available (GdkWindow *win, guint32 timestamp)
-{
-    GdkDisplay* display = gdk_window_get_display(win);
-#if GTK_CHECK_VERSION (3, 20, 0)
-    GdkSeat *seat = gdk_display_get_default_seat (display);
-#endif
-    GdkGrabStatus g = GDK_GRAB_ALREADY_GRABBED;
-    gboolean grab_failed = TRUE;
-    gint i = 0;
-
-    TRACE ("entering grab_available");
-
-    /* With a keyboard grab elsewhere, we have to wait on that to clear.
-     * So try up to 2500 times and only keep trying when the failure is
-     * already grabbed, any other failure mode will never succeed.
-     */
-    while ((i++ < 2500) && grab_failed && g == GDK_GRAB_ALREADY_GRABBED)
-    {
-#if GTK_CHECK_VERSION (3, 20, 0)
-      g = gdk_seat_grab(seat, win, GDK_SEAT_CAPABILITY_KEYBOARD, TRUE, NULL, NULL, make_window_visible, NULL);
-      if (g == GDK_GRAB_SUCCESS)
-      {
-          gdk_seat_ungrab (seat);
-          grab_failed = FALSE;
-      }
-#else
-      G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-      g = gdk_keyboard_grab (win, TRUE, timestamp);
-      if (g == GDK_GRAB_SUCCESS)
-      {
-          gdk_keyboard_ungrab(timestamp);
-          grab_failed = FALSE;
-      }
-      G_GNUC_END_IGNORE_DEPRECATIONS
-#endif
-    }
-
-    return (!grab_failed);
 }
